/*
 * Copyright (c) 2005 Pramati Technologies Private Ltd. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Pramati
 * Technologies. You shall not disclose such Confidential Information and shall
 * use it only in accordance with the terms of the source code
 * license agreement you entered into with Pramati Technologies.
 *
 */
package test;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import java.sql.SQLException;
import java.util.Set;
import java.util.List;
import java.util.Iterator;

/**
 * @author <a href="mailto:jiteshmehta@pramati.com">Jitesh Mehta</a>
 * @version $Revision$, $Date$, $Author$
 * @since Oct 13, 2006
 */
public class Main {


    public Main() {

    }

    public static void main(String[] args) throws SQLException {
        EntityManagerFactory emf = Persistence.createEntityManagerFactory("pu1");
        EntityManager em = emf.createEntityManager();
        clearDBContents(em);
        System.out.println("Starting persist of object Maps");
        A aTheOwner = null;
        int NUM_OF_INSERTS = Integer.valueOf(args[0]);
        for (int i=0; i< NUM_OF_INSERTS;i++) {
            aTheOwner = new A("name"+i);
            B bTheInverse = B.getOrCreateB(em, "b"+i, aTheOwner);
            bTheInverse.getA().add(aTheOwner); //Un-coomenting this line will throw StackOverflow
            aTheOwner.getB().add(bTheInverse);
            persist(em, aTheOwner);
        }
        System.out.println("Done ["+NUM_OF_INSERTS+"] persisting Object Maps");
        testInsert(em, aTheOwner, NUM_OF_INSERTS);
    }

    private static void clearDBContents(EntityManager em) {
        System.out.println("Main.clearDBContents");
        beginTx(em);
        List resultList = em.createNamedQuery("B.findAllB").getResultList();
        for (Iterator iterator = resultList.iterator(); iterator.hasNext();) {
            B b = (B) iterator.next();
            em.remove(b);
        }
        resultList = em.createNamedQuery("A.findAllA").getResultList();
        for (Iterator iterator = resultList.iterator(); iterator.hasNext();) {
            em.remove(((A) iterator.next()));
        }
        commitTx(em);
    }

    private static void testInsert(EntityManager em, A expected, int lastIdx) {
        System.out.println("Main.testInsert");
        final B bFromQuery = (B) em.createNamedQuery("B.findByB").setParameter("b", "b"+(lastIdx-1)).getSingleResult();
        //Get b from JDBC select query ?
        final Set<A> as = bFromQuery.getA();
        System.out.println("Main.testInsert Found ["+as.size()+"] A's from inverse side");
        boolean found = false;
        for (A a1 : as) {
            found = a1.equals(expected);
            if (found) break;
        }
        if (!found) {
            System.out.println("Could not navigate to a instance [" + expected + "] from inverse side. ");
        }
    }

    private static void commitTx(EntityManager em) {
        em.getTransaction().commit();
    }

    private static void beginTx(EntityManager em) {
        em.getTransaction().begin();
    }

    public static A persist(EntityManager em, A a) throws
            SQLException {
//        System.out.println("Main.persist");
        beginTx(em);
        em.persist(a);
        commitTx(em);
        return a;
    }
}




